#include "dataset.h"

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>


void
dataset::add_diagonal()
{
	for (uint32_t i = 0; i < m_cols.size(); i++) {
		m_rows[i].insert(i);
		m_cols[i].insert(i);
	}
}


void
dataset::add_tile(const uintvector & r, const uintvector & c)
{
	for (uint32_t i = 0; i < r.size(); i++) {
		for (uint32_t j = 0; j < c.size(); j++) {
			m_rows[r[i]].insert(c[j]);
			m_cols[c[j]].insert(r[i]);
		}
	}
}

void
dataset::remove_tile(const uintvector & r, const uintvector & c)
{
	for (uint32_t i = 0; i < r.size(); i++) {
		for (uint32_t j = 0; j < c.size(); j++) {
			m_rows[r[i]].erase(c[j]);
			m_cols[c[j]].erase(r[i]);
		}
	}
}


void
dataset::sum_rows(const uintvector &r, double mult, doublevector & out) const
{
	for (uint32_t i = 0; i < r.size(); i++) {
		for (uintset::const_iterator it = m_rows[r[i]].begin(); it != m_rows[r[i]].end(); ++it)
			out[*it] += mult;
	}
}

void 
dataset::sum_cols(const uintvector &c, double mult, doublevector & out) const
{
	for (uint32_t i = 0; i < c.size(); i++) {
		for (uintset::const_iterator it = m_cols[c[i]].begin(); it != m_cols[c[i]].end(); ++it)
			out[*it] += mult;
	}
}




uint32_t
dataset::countlines(const char *name) const
{
	uint32_t num = 0;
	std::ifstream f(name);
	std::string line;

	while (std::getline(f, line)) num++;


	return num;
}


uint32_t
dataset::countdim(const char *name) const
{
	uint32_t dim = 0;

	std::ifstream f(name);

	while (f.good()) {
		uint32_t a;
		f >> a;
		if (a + 1 > dim) dim = a + 1;
	}
	return dim;
}


dataset::dataset(const char *name)
{
	uint32_t cnt = countlines(name);
	uint32_t dim = countdim(name);

	m_cols.resize(dim);
	m_rows.resize(cnt);


	std::ifstream f(name);
	std::string line;

	uint32_t ind = 0;
	while (std::getline(f, line)) {
		//line.erase(line.find_last_not_of(" \n\r\t")+1);
		std::stringstream ss(line, std::stringstream::in);
		while (true) {
			uint32_t b;
			ss >> b;
			if (ss.fail()) break;

			//if (a > b)
				//throw std::runtime_error("Datafile is badly formatted (sparse)");

			m_cols[b].insert(ind);
			m_rows[ind].insert(b);
		}


		ind++;
	}

}

